<?php

namespace Service;

class KDSManager {

	protected $entityManager;
	protected $notificationService;
	protected $environment;
	protected $instanceManager;
	private $statisticsService;
	private $getOrigins;
	private $kdsLogs;
	private $util;
	public function __construct(\Doctrine\ORM\EntityManager $entityManager,
	\Service\Notification $notificationService,
	\Service\Statistics $statisticsService,
	\Service\GetOrigins $getOrigins,
    \Util\KDSLogs $kdsLogs,
	\Util\Util $util,
	\LIB\Environment $environment,
	\Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager)  {
		$this->entityManager = $entityManager;
		$this->notificationService = $notificationService;
		$this->statisticsService = $statisticsService;
		$this->getOrigins = $getOrigins;
        $this->kdsLogs = $kdsLogs;
		$this->util = $util;
		$this->environment = $environment;
		$this->instanceManager = $instanceManager;
	}

	public function getSales($CDFILIAL){
		$params = array(
			':CDFILIAL' => $CDFILIAL
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_SALES, $params);
	}

	public function getTriggers(){
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_TRIGGERS_KDS, $params);
	}

	public function getRequestForProductionDisplay($CDFILIAL, $NRPEDIDOFOS){
		$params = array(
			':CDFILIAL'    => $CDFILIAL,
			':NRPEDIDOFOS' => $NRPEDIDOFOS
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_REQUEST_FOR_PRODUCTION, $params);
	}

	public function getRequestItem($CDSETOR){
		$params = array(
			':CDSETOR' => $CDSETOR
		);
		$sucess = false;
        $maxTries = 30;
        $tries = 0;
        $error = null;
		$result = null;
		do {
            try {
                if(isset($CDSETOR)){
					$result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCTION_REQUESTS_BY_CDSETOR, $params);
				} else {
					$result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCTION_REQUESTS, $params);
				}
                $sucess = true;
				return $result;
            } catch (\Throwable $e) {
                $error = $e;
                $tries ++;
                sleep(0.5);
            }
        } while (!$sucess && $tries < $maxTries);

		if (!$sucess) {
            $currentDate = date("Y-m-d");
            $currentTime = date("H:i:s");
            $this->util->logException($currentDate . ' ' . $currentTime . ' ' . $error->getMessage());
        }
	}

	public function getExpedition($CDSETOR, $UPDATED = false){
        $sucess = false;
        $maxTries = 30;
        $tries = 0;
        $error = null;

		$params = array(
			':CDSETOR' => $CDSETOR
		);

		$result = array();

        do {
            try {
                if(isset($CDSETOR) && $UPDATED != false){
                    $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_EXPEDITION_BY_CDSETOR, $params);
                } else {
                    $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_EXPEDITION_UPDATED_BY_CDSETOR, $params);
                }
                $sucess = true;
            } catch (\Throwable $e) {
                $error = $e;
                $tries ++;
                sleep(0.5);
            }
        } while (!$sucess && $tries < $maxTries);

        if (!$sucess) {
            $currentDate = date("Y-m-d");
            $currentTime = date("H:i:s");
            $this->util->logException($currentDate . ' ' . $currentTime . ' ' . $error->getMessage());
        }

		if ($result) {
			$showObsCombo = $this->instanceManager->getParameter('showObsCombo');
				foreach ($result as &$element) {
					if ($showObsCombo) {
						$observacoes = null;
						$obsArray = array();
		
						$observacoes = str_replace(';', ':', $element['OBSPAI']);
						$observacoes = explode(':', $observacoes);

						foreach($observacoes as $observacao){
							$observationParams = $this->verifyObservationByName(trim($observacao));
							if(!empty($observationParams)){
								$observationParams = $observationParams[0];
								array_push($obsArray, array(
									'OBSERVACAO' => $observationParams['OBSERVACAO'],
									'DSAPELIDOOBS'=> $observationParams['DSAPELIDOOBS'],
									'NRCORSINAL' => $observationParams['NRCORSINAL'],
									'NRTEMPOPROD' => $observationParams['NRTEMPOPROD']
								));
							} else {
								if(!empty(trim($observacao))){
									array_push($obsArray, array(
										'OBSERVACAO' => trim($observacao),
										'NRCORSINAL' => 'FFFFFF'
									));
								}
							}
						}
						$element['OBSPAI'] = (json_encode($obsArray) === $element['TXPRODPED']) ? '' : $obsArray;
					} else {
						$element['OBSPAI'] = '';
					}
			}
			// Altera os dados do pedido em espera caso tenha sido transferido
			
		}
		return $result;	
	}

	public function getExpeditionItem($CDFILIAL, $NRPEDIDOFOS, $IDATRASO, $NRSEQLIBERADO, $CDSETOR){
		$params = array(
			':CDFILIAL'      => $CDFILIAL,
			':NRPEDIDOFOS'   => $NRPEDIDOFOS,
			':IDATRASO'      => $IDATRASO,
			':NRSEQLIBERADO' => $NRSEQLIBERADO,
			':CDSETOR' => $CDSETOR
		   );

		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_EXPEDITION_REQUEST_ITEM, $params);
	}

	public function insertOrder($CDFILIAL, $NRPEDIDOFOS, $DSPEDIDOFOS, $DSCHAVEPEDFOS,
				 				$DTPEDIDOFOS, $IDSTATUSPEDFOS, $DSTIPOPEDFOS, $NRMESA,
				 				$NRCOMANDA, $DSCOMANDA, $QTMAXPEDFOS, $CDSETOR,
				 				$NRPEDIDOFOSAUX, $CDLOJA, $CDSENHAPED, $NRPEDIDOFOSMSDE,
				 				$NMCONSUMIDOR, $IDATIVO){
		$params = array(
			':CDFILIAL'        => $CDFILIAL,
			':NRPEDIDOFOS'     => $NRPEDIDOFOS,
			':DSPEDIDOFOS'     => $DSPEDIDOFOS,
			':DSCHAVEPEDFOS'   => $DSCHAVEPEDFOS,
			':DTPEDIDOFOS'     => $DTPEDIDOFOS,
			':IDSTATUSPEDFOS'  => $IDSTATUSPEDFOS,
			':DSTIPOPEDFOS'    => $DSTIPOPEDFOS,
			':NRMESA'          => $NRMESA,
			':NRCOMANDA'       => $NRCOMANDA,
			':DSCOMANDA'       => $DSCOMANDA,
			':QTMAXPEDFOS'     => $QTMAXPEDFOS,
			':CDSETOR'         => $CDSETOR,
			':NRPEDIDOFOSAUX'  => $NRPEDIDOFOSAUX,
			':CDLOJA'          => $CDLOJA,
			':CDSENHAPED'      => $CDSENHAPED,
			':NRPEDIDOFOSMSDE' => $NRPEDIDOFOSMSDE,
			':NMCONSUMIDOR'    => $NMCONSUMIDOR,
			':IDATIVO'         => $IDATIVO
		);

		return $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_ORDER, $params);
	}

	public function insertOrderItem($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS, $CDPRODUTO,
			 						$QTPRODPEFOS, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND,
			 						$NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN, $TXPRODPED,
			 						$IDSITITPEDFOS, $CDSETOR, $CDGRPOCOR, $CDOCORR,
			 						$DTHREXIBICAOPROD, $DSCHAVEPEDPAI, $CDPRODUTOPAI,
			 						$CDPRODPROMOKDS, $NRSEQPRODCOMKDS, $NRTEMPOPRODIT, $NRTEMPOEXIBIT,
			 						$NRATRAPRODITPE, $NRLUGARMESAIT, $IDLIBERADO, $NRMAIORTEMPO,
			 						$IDORIGEMVENDA, $NRSEQITVENDAEST, $NRSEQCOMB){
		$params = array(
			':CDFILIAL'         => $CDFILIAL,
			':NRPEDIDOFOS'      => $NRPEDIDOFOS,
			':NRITPEDIDOFOS'    => $NRITPEDIDOFOS,
			':CDPRODUTO'        => $CDPRODUTO,
			':QTPRODPEFOS'      => $QTPRODPEFOS,
			':CDCAIXA' 		    => $CDCAIXA,
			':NRSEQVENDA'       => $NRSEQVENDA,
			':NRSEQUITVEND'     => $NRSEQUITVEND,
			':NRVENDAREST'      => $NRVENDAREST,
			':NRCOMANDA'        => $NRCOMANDA,
			':NRPRODCOMVEN'     => $NRPRODCOMVEN,
			':TXPRODPED'        => $TXPRODPED,
			':IDSITITPEDFOS'    => $IDSITITPEDFOS,
			':CDSETOR'          => $CDSETOR,
			':CDGRPOCOR'        => $CDGRPOCOR,
			':CDOCORR'          => $CDOCORR,
			':DTHREXIBICAOPROD' => $DTHREXIBICAOPROD,
			':DSCHAVEPEDPAI'    => $DSCHAVEPEDPAI,
			':CDPRODUTOPAI'     => $CDPRODUTOPAI,
			':CDPRODPROMOKDS'   => $CDPRODPROMOKDS,
			':NRSEQPRODCOMKDS'  => $NRSEQPRODCOMKDS,
			':NRTEMPOPRODIT'    => $NRTEMPOPRODIT,
			':NRTEMPOEXIBIT'    => $NRTEMPOEXIBIT,
			':NRATRAPRODITPE'   => $NRATRAPRODITPE,
			':NRLUGARMESAIT'    => $NRLUGARMESAIT,
			':IDLIBERADO'       => $IDLIBERADO,
			':NRMAIORTEMPO'     => $NRMAIORTEMPO,
			':IDORIGEMVENDA'    => $IDORIGEMVENDA,
			':NRSEQITVENDAEST'  => $NRSEQITVENDAEST,
			':NRSEQCOMB'        => $NRSEQCOMB

		);
		return $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_ORDER_ITEMS, $params);
	}

	public function insertOrderComposition($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS, $CDPRODUTO,
				 						   $QTPRODPEFOS, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND,
				 						   $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN, $TXPRODPED,
				 						   $IDSITITPEDFOS, $DSCHAVEPEDPAI, $CDSETOR, $NRLUGARMESAIT){
		$params = array(
			':CDFILIAL' 	 => $CDFILIAL,
			':NRPEDIDOFOS' 	 => $NRPEDIDOFOS,
			':NRITPEDIDOFOS' => $NRITPEDIDOFOS,
			':CDPRODUTO' 	 => $CDPRODUTO,
			':QTPRODPEFOS' 	 => $QTPRODPEFOS,
			':CDCAIXA' 		 => $CDCAIXA,
			':NRSEQVENDA' 	 => $NRSEQVENDA,
			':NRSEQUITVEND'  => $NRSEQUITVEND,
			':NRVENDAREST' 	 => $NRVENDAREST,
			':NRCOMANDA' 	 => $NRCOMANDA,
			':NRPRODCOMVEN'  => $NRPRODCOMVEN,
			':TXPRODPED' 	 => $TXPRODPED,
			':IDSITITPEDFOS' => $IDSITITPEDFOS,
			':DSCHAVEPEDPAI' => $DSCHAVEPEDPAI,
			':CDSETOR'       => $CDSETOR,
			':NRLUGARMESAIT'     => $NRLUGARMESAIT

		);
		return $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_ORDER_COMPOSITION, $params);
	}

	public function updateSaleMesa($NRPEDIDOFOS, $CDFILIAL, $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN ){
		$params = array(
			':NRPEDIDOFOS'  => $NRPEDIDOFOS,
			':CDFILIAL'     => $CDFILIAL,
			':NRVENDAREST'  => $NRVENDAREST,
			':NRCOMANDA'    => $NRCOMANDA,
			':NRPRODCOMVEN' => $NRPRODCOMVEN
		);
		return $this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_SALE_MESA, $params);
	}

	public function updateSaleVenda($NRPEDIDOFOS, $CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND ){
		$params = array(
			':NRPEDIDOFOS'  => $NRPEDIDOFOS,
			':CDFILIAL'     => $CDFILIAL,
			':CDCAIXA'      => $CDCAIXA,
			':NRSEQVENDA'   => $NRSEQVENDA,
			':NRSEQUITVEND' => $NRSEQUITVEND
		);
		return $this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_SALE_VENDA, $params);
	}

	public function doConclude($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS) {
		// change status to F
		$this->changeStatusItemRequestConclude($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS);
		// check if must expedite this item if he doesn't have a father, otherwise he will never receive status X
		$parents = $this->getItemParent($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS);
		if (empty($parents)) {
			$this->changeStatusItemRequestExpedite($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS);
			$this->updateDoneOrder($CDFILIAL, $NRPEDIDOFOS);
		}
 	}

 	public function doExpedition($CDFILIAL, $CDSETOR, $itemsArray) {
		$mustSendConclusions = false;
		$itemsToUpdate = array();
		$getSectorTypeParams = array(':CDSETOR' => $CDSETOR);
		$sectorparams = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::GET_SECTOR_DATA, $getSectorTypeParams);
		if(!empty($sectorparams) && isset($sectorparams['IDTIPOSETOR']) && $sectorparams['IDTIPOSETOR'] == 'E') {
			$this->notificationService->imprimeCupom($itemsArray[0]['NRVENDAREST'], $itemsArray[0]['CDFILIAL']);
		}
		foreach ($itemsArray as $currentItem) {
			$this->expediteItem($currentItem['CDFILIAL'], $currentItem['NRPEDIDOFOS'], $currentItem['NRITPEDIDOFOS'], $itemsToUpdate, 'S');
		}
		$this->updateExpediteItem($itemsToUpdate);
		// mustSendConclusions comentado por não ser utilizado
		//$mustSendConclusions = $this->checkIfMustSendConclusions($CDFILIAL, $CDSETOR, $itemsArray[0]['NRPEDIDOFOS']);
		$this->updatePedidofosLastAtualization($CDFILIAL, $itemsArray[0]['NRPEDIDOFOS']);
		$this->updateDoneOrder($CDFILIAL, $itemsArray[0]['NRPEDIDOFOS'], $itemsArray[0]['NRCOMANDA'], $itemsArray[0]['NRVENDAREST']);

		// foreach ($itemsArray as $currentItem) {
		// 	// Inserir rotina de insert na kds_temp_setorest aqui
		// 	// Buscar as informações necessárias para inserir na kds_temp_setorest
		// 	// rodar primeiro a rotina que elimina o excesso
		// 	$expeditions = $this->getExpeditionsInfoByNrpedidofos($CDFILIAL, $currentItem['NRPEDIDOFOS'], $currentItem['NRITPEDIDOFOS']);

		// 	foreach($expeditions as $expedition){
		// 		// A de delete eu sei que funciona preciso testar a de inserção				
		// 		try {
		// 			//code...
		// 			$this->deleteFromKdsTempSetorEst($CDFILIAL, $expedition['CDSETOR']);
		// 			$this->insertIntoKdsTempSetorEst($CDFILIAL, $expedition['CDSETOR'], $expedition['NRPEDIDOFOS'], $expedition['NRTEMPOTOTAL'], $expedition['DTHRFIMPRODUCAO']);
		// 		} catch (\Exception $error) {
		// 			//throw $th;
		// 			//O único erro que pode dar é erro de chave var_dump($error);
		// 		}
		// 	}
		// }
 	}

	public function doZoomEvent($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS) {
		$this->changeStatusZoomEvent($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS);
	}

	// public function updateExpediteItem($itemsToUpdate){
	// 	$listToUpdateWithTime = "";
	// 	$listToUpdateWithoutTime = "";
	// 	foreach ($itemsToUpdate as $currentItem) {
	// 		if ($currentItem['updateEndOfProduction'] == 'S') {
	// 			$listToUpdateWithTime = $listToUpdateWithTime . "'" . $currentItem['CDFILIAL'] . $currentItem['NRPEDIDOFOS'] . $currentItem['NRITPEDIDOFOS'] . "'". ',' ;
	// 		} else {
	// 			$listToUpdateWithoutTime = $listToUpdateWithoutTime . "'" . $currentItem['CDFILIAL'] . $currentItem['NRPEDIDOFOS'] . $currentItem['NRITPEDIDOFOS'] . "'". ',';
	// 		}
	// 	}

	// 	if (!empty($listToUpdateWithTime)) {
	// 		$listToUpdateWithTime = rtrim($listToUpdateWithTime, ",");
	// 		$this->entityManager->getConnection()->executeQuery("UPDATE ITPEDIDOFOS " .
	// 		   "SET IDSITITPEDFOS   = 'X', " .
	// 		       "DTHRFINPRODUCAO = GETDATE() " .
	// 		 "WHERE CDFILIAL + NRPEDIDOFOS + NRITPEDIDOFOS IN ($listToUpdateWithTime)");
	// 	}

	// 	if (!empty($listToUpdateWithoutTime)) {
	// 		$listToUpdateWithoutTime = rtrim($listToUpdateWithoutTime, ",");
	// 		$this->entityManager->getConnection()->executeQuery("UPDATE ITPEDIDOFOS " .
	// 		   "SET IDSITITPEDFOS   = 'X' " .
	// 		 "WHERE CDFILIAL + NRPEDIDOFOS + NRITPEDIDOFOS IN ($listToUpdateWithoutTime)");
	// 	}
	// }

	public function updateExpediteItem($itemsToUpdate){
        $listToUpdateWithTime = array();
        foreach ($itemsToUpdate as $currentItem) {
            $ITPEDIDOFOSKEY = $currentItem['CDFILIAL'] . $currentItem['NRPEDIDOFOS'] . $currentItem['NRITPEDIDOFOS'];

            $params = array(
                ':ITPEDIDOFOSKEY' => $ITPEDIDOFOSKEY,
                ':CDFILIAL' => $currentItem['CDFILIAL'],
                ':NRPEDIDOFOS' => $currentItem['NRPEDIDOFOS'] ,
                ':NRITPEDIDOFOS' => $currentItem['NRITPEDIDOFOS']
            );

            $sucess = false;
            $maxTries = 30;
            $tries = 0;
            $error = null;

            do {    
                try {
                    $this->entityManager->getConnection()->executeQuery(\Util\Query::CONCLUDE_ITPEDIDOFOS_BY_KEYS, $params);
                    $sucess = true;
                } catch (\Exception $e) {
                    $error = $e;
                    $tries ++;
                    sleep(0.5);
                }

            } while (!$sucess && $tries < $maxTries);

            if (!$sucess) {
                $currentDate = date("Y-m-d");
                $currentTime = date("H:i:s");
                $this->util->logException($currentDate . ' ' . $currentTime . ' ' . $error->getMessage());
            }
        }
    }

	public function expediteItem($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS, &$itemsToUpdate, $updateEndOfProduction) {
		// expede o item atual
		// não coloque : na frente!
		$item = array(
			'CDFILIAL'              => $CDFILIAL,
			'NRPEDIDOFOS'           => $NRPEDIDOFOS,
			'NRITPEDIDOFOS'         => $NRITPEDIDOFOS,
			'updateEndOfProduction' => $updateEndOfProduction
		);
	    array_push($itemsToUpdate, $item);

	    // verifica se item possui filhos
		$params = array(
			':CDFILIALPAI'      => $CDFILIAL,
			':NRPEDIDOFOSPAI'   => $NRPEDIDOFOS,
			':NRITPEDIDOFOSPAI' => $NRITPEDIDOFOS
		);
	    $sons = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ITEM_SONS, $params);
		if (!empty($sons)) {
			// se item possui filhos, libera um por um
			foreach ($sons as $currentSon) {
				$this->expediteItem($currentSon['CDFILIAL'], $currentSon['NRPEDIDOFOS'], $currentSon['NRITPEDIDOFOS'], $itemsToUpdate, 'N');
			}
		}
	}

	public function updateDoneOrder($CDFILIAL, $NRPEDIDOFOS, $NRCOMANDA = null, $NRVENDAREST = null){
	    // verifica se todos os itens já foram expedidos
		$params = array(
			':CDFILIAL'    => $CDFILIAL,
			':NRPEDIDOFOS' => $NRPEDIDOFOS,
			':NRCOMANDA'   => $NRCOMANDA,
			':NRVENDAREST' => $NRVENDAREST
		);
	    $pendingItems = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PENDING_PRODUCTS, $params);
	    if (empty($pendingItems)){
	    	// se todos os itens já foram expedidos, expede o pedido
			$this->entityManager->getConnection()->executeQuery(\Util\Query::EXPEDITE_ORDER, $params);
			$sectors = $this->entityManager->getConnection()->executeQuery(\Util\Query::GET_EXPEDITIONS_BY_NRPEDIDOFOS, $params);

			foreach( $sectors as $sector){
				//Atualizar quantidade da estatística aqui

                $this->statisticsService->concludeItemStatistics($params[':CDFILIAL'], $params[':NRPEDIDOFOS'], $sector['NRITPEDIDOFOS']);
				//$this->statisticsService->registrySectorConclusionQuants($CDFILIAL, $sector['CDSETOR'], $NRPEDIDOFOS);
			}
			// insere a data da expedição deste pedido nos itens em espera
			$this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_PEDIDO_KDS_TEMP_ESPERA, $params);
			// if($NRCOMANDA !== null){
			// 	$this->entityManager->getConnection()->executeQuery(\Util\Query::EXPEDITE_COMANDA, $params);
			// }
	    } else {
	    	// procura por items na expedição ou montagem que não estejam expedidos
	    	$params = array(
				':CDFILIAL'    => $CDFILIAL,
				':NRPEDIDOFOS' => $NRPEDIDOFOS
			);
		    $pendingItemsExpedition = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PENDING_PRODUCTS_EXPEDITION, $params);
		    if (!empty($pendingItemsExpedition)) {

				// procura todos os itens da produção que já foram concluídos mas não expedidos
				$params = array(
					':CDFILIAL'    => $CDFILIAL,
					':NRPEDIDOFOS' => $NRPEDIDOFOS
				);
				$pendingItemsProduction = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PENDING_PRODUCTS_PRODUCTION, $params);
				// expede todos estes itens que não têm pais
				foreach ($pendingItemsProduction as $currentItem) {
					$params = array(
						':CDFILIAL'      => $currentItem['CDFILIAL'],
						':NRPEDIDOFOS'   => $currentItem['NRPEDIDOFOS'],
						':NRITPEDIDOFOS' => $currentItem['NRITPEDIDOFOS']
					);
                    // ESSA FDP DELETA TUDO SEM PUDOR -- TENHO QUE FAZER A REVISÃO
					//$this->entityManager->getConnection()->fetchAll(\Util\Query::CHANGE_ITEM_STATUS_RELEASED, $params);
                    $this->statisticsService->concludeItemStatistics($currentItem['CDFILIAL'], $currentItem['NRPEDIDOFOS'], $currentItem['NRITPEDIDOFOS']);
				}
				// verifica se pode expedir o pedido
				$params = array(
					':CDFILIAL'    => $CDFILIAL,
					':NRPEDIDOFOS' => $NRPEDIDOFOS
				);
			    $pendingItems = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PENDING_PRODUCTS, $params);
			    if (empty($pendingItems)){
			    	// se todos os itens já foram expedidos, expede o pedido
					$this->entityManager->getConnection()->executeQuery(\Util\Query::EXPEDITE_ORDER, $params);
					// Verifica se tem itens desse pedido em espera

					//$sectorsInner = $this->entityManager->getConnection()->executeQuery(\Util\Query::GET_EXPEDITIONS_BY_NRPEDIDOFOS, $params);
					// foreach comantado por não ser utilizado
					/* foreach( $sectorsInner as $sectorInner){

						//var_dump($sectorInner);
						//Atualizar quantidade da estatística aqui
						//$this->statisticsService->registrySectorConclusionQuants($CDFILIAL, $sectorInner['CDSETOR'], $NRPEDIDOFOS);
                        //$this->statisticsService->concludeItemStatistics($params['CDFILIAL'], $params['NRPEDIDOFOS'], $params['NRITPEDIDOFOS']);
					} */
					// insere a data da expedição deste pedido nos itens em espera
					$this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_PEDIDO_KDS_TEMP_ESPERA, $params);
			    }
		    }
	    }
	}

	public function getExpeditionsByNrpedidofos($CDFILIAL, $NRPEDIDOFOS){
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'NRPEDIDOFOS' => $NRPEDIDOFOS
		);
		return $this->entityManager->getConnection()->executeQuery(\Util\Query::GET_EXPEDITION_BY_NRPEDIDOFOS, $params);
	}

	public function changeStatusItemRequestPlay($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS, $CDSETOR = null) {
        $sucess = false;
        $maxTries = 30;
        $tries = 0;
        $error = null;

		$params = array(
			':CDFILIAL'      => $CDFILIAL,
			':NRPEDIDOFOS'   => $NRPEDIDOFOS,
			':NRITPEDIDOFOS' => $NRITPEDIDOFOS,
			':CDSETOR'		 => $CDSETOR
		);
        do {
           
            try {
				if ($NRITPEDIDOFOS) {
					// se houver NRITPEDIDOFOS inicia apenas o item (utilizado em setores não agrupados)
					$this->entityManager->getConnection()->executeQuery(\Util\Query::CHANGE_STATUS_REQUEST_ITEM_PLAY, $params);
				} else {
					// inicia todos os itens do pedido (utilizado quando o setor de produção está agrupado)
					$this->entityManager->getConnection()->executeQuery(\Util\Query::CHANGE_STATUS_REQUEST_ORDER_PLAY, $params);
				}
                $sucess = true;
            } catch (\Exception $e) {
                $error = $e;
                $tries ++;
                sleep(0.5);
            }

        } while (!$sucess && $tries < $maxTries);

        if (!$sucess) {
            $currentDate = date("Y-m-d");
            $currentTime = date("H:i:s");
            $this->util->logException($currentDate . ' ' . $currentTime . ' ' . $error->getMessage());
        }
	}

	public function changeStatusItemRequestConclude($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS) {
		$params = array(
			':CDFILIAL'      => $CDFILIAL,
			':NRPEDIDOFOS'   => $NRPEDIDOFOS,
			':NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);

        $sucess = false;
        $maxTries = 30;
        $tries = 0;
        $error = null;

        do {
           
            try {
                $this->entityManager->getConnection()->executeQuery(\Util\Query::CHANGE_STATUS_REQUEST_ITEM_CONCLUDE, $params);
                $sucess = true;
            } catch (\Exception $e) {
                $error = $e;
                $tries ++;
                sleep(0.5);
            }
        
        } while (!$sucess && $tries < $maxTries);

        if (!$sucess) {
            $currentDate = date("Y-m-d");
            $currentTime = date("H:i:s");
            $this->util->logException($currentDate . ' ' . $currentTime . ' ' . $error->getMessage());
        }
	}

	public function changeStatusItemRequestExpedite($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS) {
		$params = array(
			':CDFILIAL'      => $CDFILIAL,
			':NRPEDIDOFOS'   => $NRPEDIDOFOS,
			':NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);
        $sucess = false;
        $maxTries = 30;
        $tries = 0;
        $error = null;

        do {
           
            try {
                $this->entityManager->getConnection()->executeQuery(\Util\Query::CHANGE_STATUS_REQUEST_ITEM_EXPEDITE, $params);
                $sucess = true;
            } catch (\Exception $e) {
                $error = $e;
                $tries ++;
                sleep(0.5);
            }

        } while (!$sucess && $tries < $maxTries);

        if (!$sucess) {
            $currentDate = date("Y-m-d");
            $currentTime = date("H:i:s");
            $this->util->logException($currentDate . ' ' . $currentTime . ' ' . $error->getMessage());
        }
	}

	public function changeStatusZoomEvent($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS){
		$IDSITITPEDFOS = 'X';
		$params = array(
			':CDFILIAL'      => $CDFILIAL,
			':NRPEDIDOFOS'   => $NRPEDIDOFOS,
			':NRITPEDIDOFOS' => $NRITPEDIDOFOS,
			':IDSITITPEDFOS' => $IDSITITPEDFOS
		);
		$this->entityManager->getConnection()->executeQuery(\Util\Query::CHANGE_STATUS_REQUEST_ITEM_ZOOM_EVENT, $params);

		//expedir itens filhos
		$params = array(
			':CDFILIALPAI'      => $CDFILIAL,
			':NRPEDIDOFOSPAI'   => $NRPEDIDOFOS,
			':NRITPEDIDOFOSPAI' => $NRITPEDIDOFOS
		);
		$sons = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ITEM_SONS, $params);
		if (!empty($sons)){
			foreach($sons as $son){
				$params = array(
					':CDFILIAL'      => $son['CDFILIAL'],
					':NRPEDIDOFOS'   => $son['NRPEDIDOFOS'],
					':NRITPEDIDOFOS' => $son['NRITPEDIDOFOS'],
					':IDSITITPEDFOS' => $IDSITITPEDFOS
				);
				$this->entityManager->getConnection()->executeQuery(\Util\Query::CHANGE_STATUS_REQUEST_ITEM_ZOOM_EVENT, $params);
			}
		}

		
		//as próximas operações tem como objetivo passar o status de finalizado para o front tanto
		//para o item pai do Zoom quanto para "irmãos" (filhos do mesmo pai)

		//finalizar item pai (marcar como concluído no front)
		$IDSITITPEDFOS = 'F';
		$params = array(
			':CDFILIAL'      => $CDFILIAL,
			':NRPEDIDOFOS'   => $NRPEDIDOFOS,
			':NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);
		$fathers = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ITEM_PARENT, $params);
		if (!empty($fathers)){
			$IDSITITPEDFOS = 'F';
			foreach($fathers as $father){
				$params = array(
					':CDFILIAL'      => $father['CDFILIALPAI'],
					':NRPEDIDOFOS'   => $father['NRPEDIDOFOSPAI'],
					':NRITPEDIDOFOS' => $father['NRITPEDIDOFOSPAI'],
					':IDSITITPEDFOS' => $IDSITITPEDFOS
				);
				$this->entityManager->getConnection()->executeQuery(\Util\Query::CHANGE_STATUS_REQUEST_ITEM_ZOOM_EVENT, $params);

				//finalizar filhos do item pai
				$params = array(
					':CDFILIALPAI'      => $father['CDFILIALPAI'],
					':NRPEDIDOFOSPAI'   => $father['NRPEDIDOFOSPAI'],
					':NRITPEDIDOFOSPAI' => $father['NRITPEDIDOFOSPAI'],
					':IDSITITPEDFOS' => $IDSITITPEDFOS
				);
				$sons = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ITEM_SONS, $params);
				if (!empty($sons)){
					foreach($sons as $son){
						$params = array(
							':CDFILIAL'      => $son['CDFILIAL'],
							':NRPEDIDOFOS'   => $son['NRPEDIDOFOS'],
							':NRITPEDIDOFOS' => $son['NRITPEDIDOFOS'],
							':IDSITITPEDFOS' => $IDSITITPEDFOS
						);
						$this->entityManager->getConnection()->executeQuery(\Util\Query::CHANGE_STATUS_REQUEST_ITEM_ZOOM_EVENT, $params);
					}
				}
			}
		}

	}

	public function checkOrderExistence($CHAVE, $DTPEDIDO) {
		$params = array(
			':DSCHAVEPEDFOS' => $CHAVE,
			':DTPEDIDO'		 => $DTPEDIDO
		);
		//Flagpoint var_dump para se continuar duplicando ou trazendo informacoes erradas
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::CHECK_ORDER_EXISTENCE, $params);
	}

	public function getNewCode($CDCONTADOR){
		$params = array(
			':CDCONTADOR' => $CDCONTADOR
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_NEW_CODE, $params);
	}

	public function getColourParams(){
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_COLOUR_PARAMS);
	}


	public function getProductionOrder($NRITPEDIDOFOS, $NRPEDIDOFOS){
		$params = array(
			':NRITPEDIDOFOS' => $NRITPEDIDOFOS,
			':NRPEDIDOFOS'   => $NRPEDIDOFOS
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCTION_ORDER, $params);
	}

	public function searchLocalComposition($CDFILIAL, $CDPRODUTO, $CDLOJA){
		$params = array(
			':CDFILIAL'  => $CDFILIAL,
			':CDLOJA'  => $CDLOJA,
			':CDPRODUTO' => $CDPRODUTO
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_LOCAL_COMPOSITION, $params);
	}

	public function searchPatternComposition($CDPRODUTO, $CDFILIAL, $CDLOJA){
		$params = array(
			':CDPRODUTO' => $CDPRODUTO,
			':CDFILIAL'  => $CDFILIAL,
			':CDLOJA' => $CDLOJA
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PATTERN_COMPOSITION, $params);
	}

	/* public function getAllItemObservations($CHAVEITEM){
		$sucess = false;
        $maxTries = 15;
        $tries = 0;
        $error = null;
        do {
            try {			
				$params = array(
					':CHAVEITEM' => $CHAVEITEM
				);
                $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ALL_ITEM_OBSERVATIONS, $params);
				$sucess = true;
				if ($tries < 2 && count($result) === 0) {
					$sucess = false;
					sleep(0.8);
					$tries ++;
				}
            } catch (\Exception $e) {
                $error = $e;
                $tries ++;
                sleep(0.5);
            }

        } while (!$sucess && $tries < $maxTries);

        if (!$sucess) {
            $this->util->logException($error->getMessage());
			throw new Exception('Falha na rotina getAllItemObservations');
        }

		return $result;
	} */

	public function getObservationsItemVenda ($item) {
		$sucess = false;
        $maxTries = 15;
        $tries = 0;
        $error = null;
        do {
            try {			
				$params = array(
					':CDFILIAL' => $item['CDFILIAL'],
					':CDCAIXA'	=> $item['CDCAIXA'],
					':NRSEQVENDA' => $item['NRSEQVENDA'],
					':NRSEQUITVEND' => $item['NRSEQUITVEND'],
					':CDPRODUTO' => $item['CDPRODUTO']
				);
                $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_OBSERVATIONS_VENDA, $params);
				$sucess = true;
				if ($tries < 2 && count($result) === 0) {
					$sucess = false;
					sleep(0.8);
					$tries ++;
				}
            } catch (\Exception $e) {
                $error = $e;
                $tries ++;
                sleep(0.5);
            }

        } while (!$sucess && $tries < $maxTries);

        if (!$sucess) {
            $this->util->logException($error->getMessage());
			throw new Exception('Falha na rotina getAllItemObservations');
        }

		return $result;
	}

	public function getObservationsItComanda ($item) {
		$sucess = false;
        $maxTries = 15;
        $tries = 0;
        $error = null;
        do {
            try {			
				$params = array(
					':CDFILIAL' => $item['CDFILIAL'],
					':NRVENDAREST'	=> $item['NRVENDAREST'],
					':NRCOMANDA' => $item['NRCOMANDA'],
					':NRPRODCOMVEN' => $item['NRPRODCOMVEN'],
					':CDPRODUTO' => $item['CDPRODUTO']
				);
                $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_OBSERVATIONS_COMANDA, $params);
				$sucess = true;
				if ($tries < 2 && count($result) === 0) {
					$sucess = false;
					sleep(0.8);
					$tries ++;
				}
            } catch (\Exception $e) {
                $error = $e;
                $tries ++;
                sleep(0.5);
            }

        } while (!$sucess && $tries < $maxTries);

        if (!$sucess) {
            $this->util->logException($error->getMessage());
			throw new Exception('Falha na rotina getAllItemObservations');
        }

		return $result;
	}

	public function getObservationsItemVendaBySetorProduto ($item) {
		$sucess = false;
        $maxTries = 15;
        $tries = 0;
        $error = null;
        do {
            try {			
				$params = array(
					':CDFILIAL' => $item['CDFILIAL'],
					':CDCAIXA'	=> $item['CDCAIXA'],
					':NRSEQVENDA' => $item['NRSEQVENDA'],
					':NRSEQUITVEND' => $item['NRSEQUITVEND'],
					':CDPRODUTO' => $item['CDPRODUTO'],
					':CDSETOR' => $item['CDSETOR']
				);
                $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ITEMVENDA_OBSERVATIONS_BY_SETOR_PRODUTO, $params);
				$sucess = true;
				if ($tries < 2 && count($result) === 0) {
					$sucess = false;
					sleep(0.8);
					$tries ++;
				}
            } catch (\Exception $e) {
                $error = $e;
                $tries ++;
                sleep(0.5);
            }

        } while (!$sucess && $tries < $maxTries);

        if (!$sucess) {
            $this->util->logException($error->getMessage());
			throw new Exception('Falha na rotina getAllItemObservations');
        }

		return $result;
	}

	public function getObservationsItemComandaBySetorProduto ($item) {
		$sucess = false;
        $maxTries = 15;
        $tries = 0;
        $error = null;
        do {
            try {			
				$params = array(
					':CDFILIAL' => $item['CDFILIAL'],
					':NRVENDAREST'	=> $item['NRVENDAREST'],
					':NRCOMANDA' => $item['NRCOMANDA'],
					':NRPRODCOMVEN' => $item['NRPRODCOMVEN'],
					':CDPRODUTO' => $item['CDPRODUTO'],
					':CDSETOR' => $item['CDSETOR']
				);
                $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ITCOMANDA_OBSERVATIONS_BY_SETOR_PRODUTO, $params);
				$sucess = true;
				if ($tries < 2 && count($result) === 0) {
					$sucess = false;
					sleep(0.8);
					$tries ++;
				}
            } catch (\Exception $e) {
                $error = $e;
                $tries ++;
                sleep(0.5);
            }

        } while (!$sucess && $tries < $maxTries);

        if (!$sucess) {
            $this->util->logException($error->getMessage());
			throw new Exception('Falha na rotina getAllItemObservations');
        }

		return $result;
	}


	public function verifyObservationByName($DSOCORR){
		$params = array(
			':DSOCORR' => $DSOCORR
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::VERIFY_OBSERVATION_BY_DSOCORR, $params);
	}

	/* public function getItemObservationsBySetorProduto($CHAVEITEM, $CDSETOR, $CDPRODUTO){
		$params = array(
			':CHAVEITEM' => $CHAVEITEM,
			':CDSETOR' => $CDSETOR,
			':CDPRODUTO' => $CDPRODUTO
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ITEM_OBSERVATIONS_BY_SETOR_PRODUTO, $params);
	} */

	public function getSetorProdutoObservations($CDSETOR, $CDPRODUTO){
		$params = array(
			':CDSETOR' => $CDSETOR,
			':CDPRODUTO' => $CDPRODUTO
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_SECTOR_OBSERVATIONS, $params);
	}

	public function getProductPromoComanda($CDFILIAL, $NRCOMANDA, $NRPRODCOMVEN, $NRVENDAREST){
		$params = array(
			':CDFILIAL'     => $CDFILIAL,
			':NRCOMANDA'	=> $NRCOMANDA,
			':NRPRODCOMVEN' => $NRPRODCOMVEN,
			':NRVENDAREST'  => $NRVENDAREST
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCT_PROMOTION_COMANDA, $params);
	}

	public function getProductPromoVenda($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND ){
		$params = array(
			':CDFILIAL'   => $CDFILIAL,
			':CDCAIXA'    => $CDCAIXA,
			':NRSEQVENDA' => $NRSEQVENDA,
			':NRSEQUITVEND' => $NRSEQUITVEND
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCT_PROMOTION_VENDA, $params);
	}

	public function getProductPromoRodizio($CDFILIAL, $NRCOMANDA, $NRPRODCOMVEN, $NRVENDAREST, $CDPRODUTOROD){
		$params = array(
			':CDFILIAL'     => $CDFILIAL,
			':NRCOMANDA'	=> $NRCOMANDA,
			':NRPRODCOMVEN' => $NRPRODCOMVEN,
			':NRVENDAREST'  => $NRVENDAREST,
			':CDPRODUTOROD' => $CDPRODUTOROD
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCT_PROMOTION_RODIZIO, $params);
	}

	public function sendItemsToProduction($NRITPEDIDOFOS, $DSCHAVEPEDPAI, $DTHREXIBICAOPROD){
		$params = array(
			':NRITPEDIDOFOS'    => $NRITPEDIDOFOS,
			':DSCHAVEPEDPAI'    => $DSCHAVEPEDPAI,
			':DTHREXIBICAOPROD' => $DTHREXIBICAOPROD
		);

		return $this->entityManager->getConnection()->executeQuery(\Util\Query::SEND_ITEMS_TO_PRODUCTION, $params);
	}


	public function getPatternProduct($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS){
		$params = array(
			':CDFILIAL'       => $CDFILIAL,
			':NRPEDIDOFOS'    => $NRPEDIDOFOS,
			':NRITPEDIDOFOS'  => $NRITPEDIDOFOS
		);

		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PATTERN_PRODUCT, $params);
	}

	public function searchPatternCompositionByChave($DSCHAVEPEDPAI, $CDPRODUTO){
		$params = array(
			':DSCHAVEPEDPAI' => $DSCHAVEPEDPAI,
			':CDPRODUTO'     => $CDPRODUTO
		);

		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PATTERN_COMPOSITION_BY_CHAVE, $params);
	}

	public function getMaxProductionTime($CDFILIAL, $NRPEDIDOFOS, $NRSEQLIBERADO = false){
		$params = array(
			':CDFILIAL'    => $CDFILIAL,
			':NRPEDIDOFOS' => $NRPEDIDOFOS
		);
		if($NRSEQLIBERADO == false){
			return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_MAX_PRODUCTION_TIME, $params);
		} else {
			return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_MAX_PRODUCTION_TIME_NRATRAPRODITPED, $params);
		}
	}

	public function getProductSector($CDPRODUTO, $CDFILIAL, $CDLOJA){
		$params = array(
			':CDFILIAL'  => $CDFILIAL,
			':CDPRODUTO' => $CDPRODUTO,
			':CDLOJA' => $CDLOJA
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCT_SECTOR, $params);
	}

	public function getProductSectorByType($CDPRODUTO, $CDFILIAL, $CDLOJA, $IDTIPOSETOR) {
		$params = array(
			':CDFILIAL'    => $CDFILIAL,
			':CDPRODUTO'   => $CDPRODUTO,
			':CDLOJA'      => $CDLOJA,
			':IDTIPOSETOR' => $IDTIPOSETOR
		);

		$expedition = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCT_SECTOR_BY_TYPE_STORE, $params);

		if (count($expedition) == 0) {
			$params = array(
				':CDFILIAL' => $CDFILIAL,
				':CDLOJA'   => $CDLOJA
			);
			$countSectorStore = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_SECTOR_STORE_COUNT, $params);
			if (count($countSectorStore) == 0) {
				$params = array(
					':CDFILIAL'    => $CDFILIAL,
					':CDPRODUTO'   => $CDPRODUTO,
					':IDTIPOSETOR' => $IDTIPOSETOR
				);

				$expedition = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCT_SECTOR_BY_TYPE, $params);
				
			}
		}
		return $expedition;
	}

	public function redirectOrderSector($CDFILIAL, $item, $IDORIGEMVENDA){
		/* Rerireciona o pedido com base no IDORIGEMVENDA*/
		$IDORIGEMVENDA = $this->getOrigins->manageOrigin($IDORIGEMVENDA, "system", "modality");
		if(count($item) > 0){
			foreach ($item as &$it){
				if($it['IDHABREDIREC'] == 'S'){
					$params = array(
						'CDFILIAL' => $CDFILIAL,
						'CDSETOR' => $it['CDSETOR']
					);

					$redirectSectors = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_REDIRECT_SECTOR, $params);

					array_filter($redirectSectors, function($redirect) use ($IDORIGEMVENDA, &$it){
						$origem = $this->getOrigins->manageOrigin($redirect['IDORIGEMVENDA'], "system", "modality") == $IDORIGEMVENDA;
						if($origem && $redirect['IDHABILITADO'] == 'S' && $it['IDTIPOSETOR'] == $redirect['IDTIPOSETOR']){
							$it['CDSETOR'] = $redirect['CDSETORRED'];
						}
					});
				}
			}
		}
		return $item;
	}

	public function checkStatusProduction($DSCHAVEPEDPAI){
		$params = array(
			':DSCHAVEPEDPAI' => $DSCHAVEPEDPAI
		);

		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CHECK_PRODUCTION, $params);
	}

	public function checkStatusDone($DSCHAVEPEDPAI){
		$params = array(
			':DSCHAVEPEDPAI' => $DSCHAVEPEDPAI
		);

		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CHECK_DONE, $params);
	}

	public function getSubProducts($DSCHAVEPEDPAI){
		$params = array(
			':DSCHAVEPEDPAI' => $DSCHAVEPEDPAI
		);

		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_SUBPRODUCTS, $params);
	}

	public function getChildProducts($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS) {
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'NRPEDIDOFOS' => $NRPEDIDOFOS,
			'NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);
		$spawn = array();
		$sons = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CHILD_PRODUCTS, $params);
		$spawn = array_merge($spawn, $sons);
		foreach ($sons as $son) {
			$grandSons = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CHILD_PRODUCTS, $son);
			$spawn = array_merge($spawn, $grandSons);
		}
		return $spawn;
	}

	public function checkChildDone($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS, $IDTIPOSETOR) {
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'NRPEDIDOFOS' => $NRPEDIDOFOS,
			'NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);
		$spawn = array();
		$sons = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CHILD_PRODUCTS, $params);
		foreach ($sons as $son) {
			if(($son['IDSITITPEDFOS'] == 'F' && $son['IDTIPOSETOR'] == 'P')|| $son['IDSITITPEDFOS'] == 'X'){
				$spawn = array_merge($spawn, array($son));
			}
			$grandSons = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CHILD_DONE, $son);
			$spawn = array_merge($spawn, $grandSons);
		}
		return $spawn;
	}

	public function checkChildProduction($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS) {
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'NRPEDIDOFOS' => $NRPEDIDOFOS,
			'NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);
		$spawn = array();
		$sons = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CHILD_PRODUCTS, $params);
		foreach ($sons as $son) {
			if($son['IDSITITPEDFOS'] == 'P') {
				$spawn = array_merge($spawn, array($son));
			}
			$grandSons = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CHILD_PRODUCTION, $son);
			$spawn = array_merge($spawn, $grandSons);
		}
		return $spawn;
	}

	public function getObservation($CDPRODUTO){
		$params = array(
			':CDPRODUTO' => $CDPRODUTO
		);

		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_OBSERVATION, $params);
	}

	public function getProductBySeqOrder($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS) {
		$params = array(
			':CDFILIAL' 	 =>$CDFILIAL,
			':NRPEDIDOFOS' 	 =>$NRPEDIDOFOS,
			':NRITPEDIDOFOS' =>$NRITPEDIDOFOS
		);

		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCT_BY_SEQ_ORDER, $params);
	}

	public function clearRequestsScreen($CDFILIAL, $CDSETOR, $hoursNumber) {
		$params = array(
			':CDFILIAL' => $CDFILIAL,
			':CDSETOR' => $CDSETOR,
			':NRHORA' => $hoursNumber
		);
		$this->entityManager->getConnection()->executeQuery(\Util\Query::CLEAN_PEDIDOFOS_FROM_LOJA_BY_CDSETOR, $params);
	}

	public function updatePedidofosLastAtualization($CDFILIAL, $NRPEDIDOFOS) {
		$params = array(
			':CDFILIAL'    => $CDFILIAL,
			':NRPEDIDOFOS' => $NRPEDIDOFOS
		);
		$this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_DTULTATU_PEDIDOFOS, $params);
	}

	public function getPedidofosByVenda($CDFILIAL, $NRSEQVENDA, $NRSEQUITVEND){
		$params = array(
			':CDFILIAL' => $CDFILIAL,
			':NRSEQVENDA' => $NRSEQVENDA,
			':NRSEQUITVEND' => $NRSEQUITVEND
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PEDIDOFOS_BY_VENDA, $params);
	}

	public function getNewTemporaryCode($CDFILIAL, $NRPEDIDOFOS){
		$params = array(
			':CDFILIAL' => $CDFILIAL,
			':NRPEDIDOFOS' => $NRPEDIDOFOS
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_NEW_TEMPORARY_CODE, $params);
	}

	public function getProductPromoBalcaoSemVenda($CDFILIAL, $NRPEDIDOFOS){
		$params = array(
			':CDFILIAL'   => $CDFILIAL,
			':NRPEDIDOFOS' => $NRPEDIDOFOS
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCT_SEM_VENDA, $params);
	}

	public function checkIfKeyExists($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS){
		$params = array(
			':CDFILIAL' => $CDFILIAL,
			':NRPEDIDOFOS' => $NRPEDIDOFOS,
			':NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);
		$count = $this->entityManager->getConnection()->fetchAll(\Util\Query::CHECK_IF_KEY_EXISTS, $params);
		if (empty($count)) {
			return true;
		} else {
			return false;
		}
	}

	public function checkIfITPEDIDOFOSPAIExists($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS, $NRSEQITPEDPAI){
		$params = array(
			':CDFILIAL' => $CDFILIAL,
			':NRPEDIDOFOS' => $NRPEDIDOFOS,
			':NRITPEDIDOFOS' => $NRITPEDIDOFOS,
			':NRSEQITPEDPAI' => $NRSEQITPEDPAI
		);
		$count = $this->entityManager->getConnection()->fetchAll(\Util\Query::CHECK_IF_ITPEDIDOFOSPAI_EXISTS, $params);
		if (empty($count)) {
			return true;
		} else {
			return false;
		}
	}

	public function findOldRequests($CDFILIAL){
		$params = array(
			':CDFILIAL'   => $CDFILIAL
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::FIND_OLD_REQUESTS, $params);
	}

	public function deleteItemPedidofos($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS){
		$params = array(
			':CDFILIAL' => $CDFILIAL,
			':NRPEDIDOFOS' => $NRPEDIDOFOS,
			':NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::DELETE_ITPEDIDOFOS, $params);
	}

	public function deletePedidofos($CDFILIAL, $NRPEDIDOFOS){
		$params = array(
			':CDFILIAL' => $CDFILIAL,
			':NRPEDIDOFOS' => $NRPEDIDOFOS
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::DELETE_PEDIDOFOS, $params);
	}

	public function getSetorProdutoFilial($CDPRODUTO, $CDFILIAL, $IDTIPOSETOR, $CDSETOR){
		$params = array(
			':CDPRODUTO' => $CDPRODUTO,
			':CDFILIAL' => $CDFILIAL,
			':IDTIPOSETOR' => $IDTIPOSETOR,
			':CDSETOR' => $CDSETOR
		);

		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUTO_FILIAL, $params);
	}

	public function insertRelation($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS, $NRSEQITPEDPAI, $CDFILIALPAI, $NRPEDIDOFOSPAI, $NRITPEDIDOFOSPAI){
		$params = array(
			'CDFILIAL'         => $CDFILIAL,
			'NRPEDIDOFOS'      => $NRPEDIDOFOS,
			'NRITPEDIDOFOS'    => $NRITPEDIDOFOS,
			'NRSEQITPEDPAI'    => $NRSEQITPEDPAI,
			'CDFILIALPAI'      => $CDFILIALPAI,
			'NRPEDIDOFOSPAI'   => $NRPEDIDOFOSPAI,
			'NRITPEDIDOFOSPAI' => $NRITPEDIDOFOSPAI
		);

		return $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_ITPEDIDOFOSPAI, $params);
	}

	public function getProductByCode($CDPRODUTO){
		$params = array(
			'CDPRODUTO' => $CDPRODUTO
		);

		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCT_BY_CODE, $params);
	}

	public function insertOrderRelation($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS, $NRSEQITPEDREL,
	                                    $NRVENDAREST, $NRCOMANDA, $NRPRODCOMVEN, $QTPRODPEFOS,
	                                    $IDSITITPEDFOS){
		$params = array(
			'CDFILIAL'      => $CDFILIAL,
			'NRPEDIDOFOS'   => $NRPEDIDOFOS,
			'NRITPEDIDOFOS' => $NRITPEDIDOFOS,
			'NRSEQITPEDREL' => $NRSEQITPEDREL,
			'NRVENDAREST'   => $NRVENDAREST,
			'NRCOMANDA'     => $NRCOMANDA,
			'NRPRODCOMVEN'  => $NRPRODCOMVEN,
			'QTPRODPEFOS'   => $QTPRODPEFOS,
			'IDSITITPEDFOS' => $IDSITITPEDFOS
		);
		return $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_ITPEDIDOFOSREL, $params);
	}

	public function getItemsRelations($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS) {
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'NRPEDIDOFOS' => $NRPEDIDOFOS,
			'NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);
		$result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ITEMS_RELATION, $params);
		return $result;
	}

	public function getExpeditionAndMountListFromItPedidofos(){
		$params = [];
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_EXPEDITIONS_AND_MOUNTS_CDSETOR_FROM_ITPEDIDOFOS, $params);
	}

	public function cancelItem($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS) {
		$params = array(
			'CDFILIAL'      => $CDFILIAL,
			'NRPEDIDOFOS'   => $NRPEDIDOFOS,
			'NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);
		return $this->entityManager->getConnection()->executeQuery(\Util\Query::CANCEL_ITEM, $params);
	}

	public function changeItemQuantity($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS, $QTPRODPEFOS) {
		$params = array(
			'CDFILIAL'      => $CDFILIAL,
			'NRPEDIDOFOS'   => $NRPEDIDOFOS,
			'NRITPEDIDOFOS' => $NRITPEDIDOFOS,
			'QTPRODPEFOS'   => $QTPRODPEFOS
		);
		return $this->entityManager->getConnection()->executeQuery(\Util\Query::CHANGE_ITEM_QUANTITY, $params);
	}

	public function getItemIntervals($CDPRODUTO, $CDSETOR) {
		$params = array(
			'CDPRODUTO' => $CDPRODUTO,
			'CDSETOR'   => $CDSETOR
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ITEM_INTERVALS, $params);
	}

	public function releaseItem($NRSEQLIBERADO, $CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS) {
		$params = array(
			'NRSEQLIBERADO' => $NRSEQLIBERADO,
			'CDFILIAL'      => $CDFILIAL,
			'NRPEDIDOFOS'   => $NRPEDIDOFOS,
			'NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);
		return $this->entityManager->getConnection()->executeQuery(\Util\Query::RELEASE_ITEM, $params);
	}

	public function getItemParent($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS) {
		$params = array(
			'CDFILIAL'      => $CDFILIAL,
			'NRPEDIDOFOS'   => $NRPEDIDOFOS,
			'NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ITEM_PARENT, $params);
	}

	public function getItemCascade($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS) {
		$params = array(
			'CDFILIAL'      => $CDFILIAL,
			'NRPEDIDOFOS'   => $NRPEDIDOFOS,
			'NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ITEM_CASCADE, $params);
	}

	public function checkIfMustSendConclusions($CDFILIAL, $CDSETOR, $NRPEDIDOFOS) {
		$params = array(
			'CDFILIAL'    => $CDFILIAL,
			'CDSETOR'     => $CDSETOR,
			'NRPEDIDOFOS' => $NRPEDIDOFOS
		);
		$nonExpeditedItems = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_IF_MUST_SEND_CONCLUSIONS, $params);
		if (count($nonExpeditedItems) > 0) {
			return false;
		} else {
			return true;
		}
	}

	public function changeItemStatusReleased($CDFILIAL, $NRPEDIDOFOS, $NRITPEDIDOFOS) {
		$params = array(
			'CDFILIAL'      => $CDFILIAL,
			'NRPEDIDOFOS'   => $NRPEDIDOFOS,
			'NRITPEDIDOFOS' => $NRITPEDIDOFOS
		);
		return $this->entityManager->getConnection()->executeQuery(\Util\Query::CHANGE_ITEM_STATUS_RELEASED, $params);
	}

	public function getMappedKeys() {
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_MAPPED_KEYS);
	}

	private function logExceptionWSService($info){
        //testa se a pasta existe e cria caso não exista
        $folder = substr(__DIR__, 0, 1) . ":\\TEKNISA\\kds\\logs\\";
        if (!file_exists($folder)) {
            mkdir($folder, 0777, true);
        }
        //--grava o log
        $line = date("d-m-Y H:i:s") . " - " . $info . "\n";
        $drive = substr(__DIR__, 0, 1);
        $date = date("dmY");
        file_put_contents($drive . ":\\TEKNISA\\kds\\logs\\logExceptionWSService" . $date . ".txt", $line, FILE_APPEND);
    }

    public function getDefaultFilial(){
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_DEFAULT_FILIAL);
	}

	public function getFixedSummaryItems($CDFILIAL, $CDSETOR){
		$params = array(
			':CDFILIAL' => $CDFILIAL,
			':CDSETOR'  => $CDSETOR
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_FIXED_SUMMARY_ITEMS, $params);
	}

	public function getLineStatus( $CDSETOR){
		$params = array(
			':CDSETOR'  => $CDSETOR
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_STATUS_LINHAPROD, $params);
	}

	public function updateLineStatus($CDSETOR, $CDFILIAL){
		$params = array(
			':CDSETOR'  => $CDSETOR,
			':CDFILIAL'  => $CDFILIAL,
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::UPDATE_STATUS_LINHAPROD, $params);
	}

	public function setLinePercentage( $CDSETOR, $CDFILIAL, $PERCENT){
		$params = array(
			':CDSETOR'  => $CDSETOR,
			':CDFILIAL'  => $CDFILIAL,
			':NRPERCDEMANDA'  => $PERCENT
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::UPDATE_PERCENT_DEMAND, $params);
	}

	public function updateDtUltAtuItPedidoFosByNrPedidoFos($CDFILIAL, $NRPEDIDOFOS) {
		$sucess = false;
        $maxTries = 30;
        $tries = 0;
        $error = null;

		$params = array(
			':CDFILIAL' => $CDFILIAL,
			':NRPEDIDOFOS'  => $NRPEDIDOFOS
		);

		do {
            
			try {
				$this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_DTULTATU_ITPEDIDOFOS_BY_NRPEDIDOFOS, $params);
				$sucess = true;
			} catch (\Exception $e) {
				$error = $e;
				$tries ++;
				sleep(0.5);
			}

		} while (!$sucess && $tries < $maxTries);
		
		if (!$sucess) {
			$this->util->logException($error->getMessage());
		}
		/* $items = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_DTULTATU_ITPEDIDOFOS_BY_NRPEDIDOFOS, $params);

		foreach ($items as $key => $value) {
			$paramsItem = array(
				':CDFILIAL' => $value['CDFILIAL'],
				':NRPEDIDOFOS'  => $value['NRPEDIDOFOS'],
				':NRITPEDIDOFOS' => $value['NRITPEDIDOFOS']
			);
			
            do {
            
                try {
                    $this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_DTULTATU_ITPEDIDOFOS_BY_NRPEDIDOFOS, $paramsItem);
                    $sucess = true;
                } catch (\Exception $e) {
                    $error = $e;
                    $tries ++;
                    sleep(0.5);
                }

            } while (!$sucess && $tries < $maxTries);
            
            if (!$sucess) {
                $currentDate = date("Y-m-d");
                $currentTime = date("H:i:s");
                $this->util->logException($currentDate . ' ' . $currentTime . ' ' . $error->getMessage());
            }

		} */
	}

	public function checkIfCanDoneMount($NRPEDIDOFOS){
		$params = array(
			':NRPEDIDOFOS' => $NRPEDIDOFOS
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CDSETOR_PRIORM_BY_NRPEDIDOFOS, $params);
	}

	public function getStatisticsFromOneClient($CDFILIAL, $CDSETOR){
		// envia estatisticas
		$params = array(
			':CDFILIAL' => $CDFILIAL, // T de todos
			':CDSETOR'  => $CDSETOR, // T de todos
            ':IDTIPOSETOR' => 'E'
		);
		$newStatistics = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_STATISTICS, $params);
		return $newStatistics;
	}

	public function getProductionLineActive($CDFILIAL){
		$params = array(':CDFILIAL' => $CDFILIAL);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_LINHAPROD, $params);
	}

	public function getCorrelationatedSectors($CDFILIAL){
		$params = array(':CDFILIAL' => $CDFILIAL);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CORRELATIONATED_SECTORS, $params);
	}

	public function getCorrelationatedSectorsCode($CDFILIAL, $CDSETOR){
		$params = array(
			':CDFILIAL' => $CDFILIAL,
			':CDSETOR' => $CDSETOR
		);
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CDSETORCORRESP, $params);
	}


	public function getProductionLineSectorByCdSetor($CDFILIAL, $CDSETOR){
		$params = array(
			':CDFILIAL' => $CDFILIAL,
			':CDSETOR' => $CDSETOR
		);
		
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_LINHAPRODSETOR, $params);
	}

	public function deleteFromKdsTempSetorEst($CDFILIAL, $CDSETOR){
		$params = array(
			':CDFILIAL'   => $CDFILIAL,
			':CDSETOR'    => $CDSETOR
		);

		return $this->entityManager->getConnection()->executeQuery(\Util\Query::DELETE_FROM_KDS_TEMP_SETOREST, $params);
	}

	public function insertIntoKdsTempSetorEst($CDFILIAL, $CDSETOR, $NRPEDIDOFOS, $NRTEMPOTOTAL, $DTHRFIMPRODUCAO){

		$params = array(
			':CDFILIAL'   => $CDFILIAL,
			':CDSETOR'    => $CDSETOR,
			':NRPEDIDOFOS' => $NRPEDIDOFOS,
			':NRTEMPOTOTAL' => $NRTEMPOTOTAL,
			':DTHRFIMPRODUCAO' => $DTHRFIMPRODUCAO
		);

        // Esse aqui não está sendo chamado de nenhum lugar
		return $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_INTO_KDS_TEMP_SETOREST, $params);
	}

	public function getExpeditionsInfoByNrpedidofos($CDFILIAL, $NRPEDIDOFOS){
		$params = array(
			':CDFILIAL'   => $CDFILIAL,
			':NRPEDIDOFOS' => $NRPEDIDOFOS
		);

		//return $this->entityManager->getConnection()->executeQuery(\Util\Query::GET_EXPEDITIONS_INFO_BY_NRPEDIDOFOS, $params);
	}

	public function getSectorType($CDSETOR){
		$params = array('CDSETOR' => $CDSETOR);

		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_SECTOR_TYPE, $params);
		
	}

	public function updateNotificationTime($CDFILIAL, $NRPEDIDOFOS){
		try {
			$params = array(
				'CDFILIAL' => $CDFILIAL,
				'NRPEDIDOFOS'  => $NRPEDIDOFOS
			);
			$this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_ORDER_DTHRPRIEXPED, $params);
            $this-> kdsLogs-> log("UPDATE_ORDER_DTHRPRIEXPED foi chamada com os parametros: ", $params, dirname(__FILE__).'\\'.basename(__FILE__));

		}catch(\Exception $error){
			var_dump($error);
		}		
	}

	public function refreshOrderItem($NRID = NULL){
		try {
			$params = array(
				'NRID'  => $NRID
			);
			$this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_ITPEDIDOFOS_BY_NRVENDAREST, $params);
		}catch(\Exception $error){
			var_dump($error);
		}		
	}

    public function verificaHabAgrupProd($setor,$filial){
		$params = array(
			':CDSETOR'  => $setor,
			':CDFILIAL' => $filial
		);

		return $this->entityManager -> getConnection()->fetchAll(\Util\Query::VERIFY_IDHABAGRUPPROD_SETORFILIAL, $params)[0]['IDHABAGRUPPROD'];
	}
	

	public function verificaHabAgrup($filial){
		$params = array(
			':CDFILIAL' => $filial
		);

		return $this->entityManager -> getConnection()->fetchAll(\Util\Query::VERIFY_IDHABAGRUPPROD, $params);
	}

	public function verificaPicklistPorBotao($CDSETOR) {
		$params = array(
			':CDSETOR' => $CDSETOR
		);

		try {
			return $this->entityManager -> getConnection()->fetchAll(\Util\Query::GET_IDHABPICKLISTIMPBTN_BY_CDSETOR, $params)[0]['IDHABPICKLISTIMPBTN'] == "S";
		} catch (\Throwable $th) {
			$this->util->logException($th->getMessage());
			return false;
		}
	}

	public function getProdsEntrada ($CDFILIAL) {
		$params['CDFILIAL'] = $CDFILIAL;
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODS_ENTRADA, $params);
	}

	public function getProdsExcecao ($CDFILIAL) {
		$params['CDFILIAL'] = $CDFILIAL;
		return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODS_EXCECAO, $params);
	}

	public function insertPedidoKdsTempEspera($params) {
		$this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_PEDIDO_KDS_TEMP_ESPERA, $params);
	}

}
