var ZeedhiDirectives = angular.module("ZeedhiDirectives");

function loadImage(imageUrl, callback) {
	if (imageUrl) {
		$.ajax({
			url: imageUrl,
			cache: true,
			processData: false
		}).done(callback);
	}
}

ZeedhiDirectives.directive("zhProductGrid", function() {
	return {
		link: function($scope, element) {
			var cart = element.find('div.cart');
			var header = element.find('div.product-grid-header');

			element.on("scroll.product-grid", function() {
				cart.css('margin-top', element.scrollTop());
				header.css('margin-top', element.scrollTop());
			});

			header.find('.header-shortcut').on("change", function(e) {
				var selectedItem = $(e.currentTarget).find(':selected').val();
				if (selectedItem) {
					var group = element.find('ul.product-groups > li[data-group-code="'+selectedItem+'"]');
					var groupTop = group.position().top;
					element.animate({ scrollTop: groupTop - 60 }, 300);
				}
			});

			var cartItems = cart.find('ul.cart-items');
			cartItems.on("mousewheel DOMMouseScroll", function(e, delta) {
				if (cartItems.height() < cartItems[0].scrollHeight) {
					if ((e.originalEvent.deltaY < 0 && cartItems.scrollTop() === 0) ||
						  (e.originalEvent.deltaY > 0 && cartItems.scrollTop() + cartItems.height() == cartItems[0].scrollHeight)) {
						return false;
					}
				}
			});

			$scope.toggleGroup = function(e) {
				var group = $(e.currentTarget.parentElement);
				if (group.hasClass('opened')) {
					group.css('height', (group.hasClass('no-image') ? 32 : 112) + group.find('ul.product-items').innerHeight());
					group.removeClass('opened');
					group.animate({'height': 32}, 300);
				} else {
					group.css('height', 32);
					group.addClass('opened');
					group.animate({'height': (group.hasClass('no-image') ? 32 : 112) + group.find('ul.product-items').innerHeight()}, 300, function() {
						group.css('height', 'auto');
					});
				}
			};

			$scope.toggleCartItems = function(e) {
				var cart = $(e.currentTarget).parents('.cart').eq(0);
				cart.toggleClass('opened');
			};

			$scope.loadGroupImage = function(index, imageUrl) {
				var group = element.find('ul.product-groups > li').eq(index);
				loadImage(imageUrl, function() {
					var element = group.find('.group-img span');
					element.css("background-image",'url('+imageUrl+')').fadeIn();
					group.removeClass('no-image');
				});
			};

			$scope.$watch(function() { return $scope.widget.searchCriteria; }, function(val) {
				// force groups to be opened during searching
				if (val) { element.find('ul.product-groups > li').addClass('opened').css('height', 'auto'); }
			});
		}
	};
});

ZeedhiDirectives.directive("zhProductGridDetailImage", function() {
	return {
		link: function($scope, element, attrs) {
			var imageUrl = $scope.$eval(attrs.zhProductGridDetailImage);
			loadImage(imageUrl, function() {
				var span = element.find('span');
				span.css("background-image",'url('+imageUrl+')').fadeIn();
				element.parent().removeClass('no-image');
			});
		}
	};
});

ZeedhiDirectives.directive("zhProductGridDetailTab", function(templateManager, $timeout) {
	return {
		link: function($scope, element) {
			function updateScrollIndicators() {
				if (this instanceof IScroll) {
					element.removeClass('overflow-right');
					element.removeClass('overflow-left');
					if ((element.width() - this.x) < (element.find('.tab-scroller').width() - 5)) {
						element.addClass('overflow-right');
					}
					if (this.x < 0) {
						element.addClass('overflow-left');
					}
				} else {
					if (element.width() < element.find('.tab-scroller').width()) {
						element.addClass('overflow-right');
					}
				}
			}

			$timeout(function() {
				var scroll = new IScroll(element[0], {
					eventPassthrough: true,
					scrollX: true,
					scrollY: false,
					preventDefault: false
				});
				scroll.on('scrollEnd', updateScrollIndicators);
				updateScrollIndicators();

				element.find('.overflow-right').on('click', function() {
					scroll.scrollBy(-100, 0, 200, IScroll.utils.ease.quadratic);
				});

				element.find('.overflow-left').on('click', function() {
					scroll.scrollBy(100, 0, 200, IScroll.utils.ease.quadratic);
				});

				templateManager.scrollers = templateManager.scrollers || [];
				templateManager.scrollers.push(scroll);
			});
		}
	};
});