var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive("zhColorPalette", function($timeout, ScreenService) {
	return {
		restrict: 'A',
		link: function($scope, $element) {
			var widget = $scope.widget;
			var field = widget.parentField;
			widget.restrictions = widget.restrictions || {};

			if(widget.currentRow.length) {
				$scope.selectedColors = widget.currentRow.map(function(color) {
					return tinycolor(color).toHexString();
				});
			} else if(field && !!field.widget.currentRow && !!field.widget.currentRow[field.name] && field.widget.currentRow[field.name].length) {
				$scope.selectedColors = field.widget.currentRow[field.name].map(function(color) {
					return tinycolor(color).toHexString();
				});
			} else {
				$scope.selectedColors = [];
			}

			function isColorSelected(color) {
				return $scope.selectedColors.indexOf(color) != -1;
			}

			function selectColor(color) {
				if(isColorSelected(color)) {
					$scope.selectedColors.splice($scope.selectedColors.indexOf(color), 1);
				} else if($scope.selectedColors.length < $scope.widget.restrictions.maxColors) {
					$scope.selectedColors.push(color);
				} else if(!$scope.widget.restrictions.maxColors) {
					$scope.selectedColors.push(color);
				}
			}

			function getPlural(amount) {
				return amount > 1 ? " colors" : " color";
			}

			function getHelperText() {
				var phrase;
				var minColors = widget.restrictions.minColors;
				var maxColors = widget.restrictions.maxColors;

				if(widget.helperText) {
					phrase = widget.helperText;
				} else if((minColors && maxColors) && (minColors === maxColors)) {
					phrase = "Select " + maxColors + getPlural(maxColors);
				} else if(minColors && maxColors) {
					phrase = "Select "  + minColors + " to " + maxColors + getPlural(Math.max(minColors, maxColors));
				} else if(minColors) {
					phrase = "Select at least " + minColors + getPlural(minColors);
				} else if(maxColors) {
					phrase = "Select up to " + maxColors + getPlural(maxColors);
				} else {
					phrase = "";
				}

				return phrase;
			}

			$scope.isColorSelected = isColorSelected;
			$scope.selectColor = selectColor;
			$scope.getHelperText = getHelperText;

			$scope.$watchCollection("selectedColors", function(selectedColors) {
				var selectedColorObjects = selectedColors.map(function(color) {
					return tinycolor(color);
				});

				$scope.widget.setCurrentRow(selectedColorObjects);
			});

		}
	};
});
