describe("EventEngine ", function() {
	var updateTemplate = false;
	/*jslint evil: true */
	var eventEngine = new EventEngine(
		new EventFactory(new MethodBuilder({}, {
			$eval: function(a, b) {
				a = "var args = b.args;" + a;
				return eval(a, b);
			}
		}), {}),
		new EventHandler({}, {}, console),
		{
			updateTemplate: function() {
				updateTemplate = true;
			}
		}, 
		{
			i18n: function(a) {
				return a;
			}
		},
		{logEvent:function() {}}
	);

	beforeEach(function() {
		updateTemplate = false;
	});

	it("bindEvents should create a method named afterMoveRow when I bind WidgetAfterMoveRow",
		function() {
			var obj = {};
			eventEngine.bindEvents(obj, [{
				"name": "WidgetAfterMoveRow",
				"code": ";"
			}], function() {}, function() {});
			expect(obj.afterMoveRow).not.toEqual(undefined);
	});
	it("bindEvent should create a method named afterMoveRow when I bind WidgetAfterMoveRow",
		function() {
			var obj = {};
			eventEngine.bindEvent(obj, "WidgetAfterMoveRow",";");
			expect(obj.afterMoveRow).not.toEqual(undefined);
	});		
	it("bindEvents should create a method named FieldOnLongTap when I bind FieldOnTap or FieldOnLongTap",
		function() {
			expect(function() {
				var obj = {
					on: function(n, fn) {
						obj[n] = fn;
					}
				};
				eventEngine.bindEvents(obj, [{
					"name": "FieldOnLongTap",
					"code": ";"
				}], function() {}, function() {});
				expect(obj.longtap).not.toEqual(undefined);
				obj.longtap({
					isPropagationStopped: function() {
						return false;
					}
				});
			}).not.toThrow();
		});
	it("bindEvents should throw an error when event doesn't exists!", function() {
		var obj = {};
		expect(function() {
			eventEngine.bindEvents(obj, [{
				"name": "WidgetWhenGodDesires",
				"code": ";"
			}], function() {}, function() {});
		}).toThrow();
	});

	it(
		"bindEvents should create a method named afterMoveRow when I bind WidgetAfterMoveRow and if afterMoveRow caled with a function it add this function as a new listener and not call the method itself",
		function() {
			var obj = {};
			var data = null;
			var data2 = null;
			eventEngine.bindEvents(obj, [{
				"name": "WidgetAfterMoveRow",
				"code": ";"
			}], function() {}, function() {});
			expect(obj.afterMoveRow).not.toEqual(undefined);
			obj.afterMoveRow(function() {
				data = "inAfterMoveRow";
			});
			obj.afterMoveRow(function() {
				data2 = "inAfterMoveRow2";
			});
			expect(data).toBe(null);
			expect(data2).toBe(null);
			obj.afterMoveRow();
			expect(data).toBe("inAfterMoveRow");
			expect(data2).toBe("inAfterMoveRow2");
			expect(updateTemplate).toEqual(true);
		});

	it("bindEvents should use existent method if there", function() {
		var check = null;
		var handler = function(fn) {
			if (fn) {
				check = "handlerCalled";
			}
		};
		var obj = {
			afterMoveRow: handler
		};
		eventEngine.bindEvents(obj, [{
			"name": "WidgetAfterMoveRow",
			"code": ";"
		}], function() {}, function() {});
		expect(obj.afterMoveRow).toEqual(handler);
		expect(check).toEqual("handlerCalled");

	});

	it(
		"bindEvents should use existent method if there and function sended by parameter needs to be a handler to execute code",
		function() {
			var check = null;
			var handler = function(fn) {
				if (fn) {
					fn();
				}
			};

			var obj = {};
			eventEngine.createEvent(obj, 'afterMoveRow');
			eventEngine.bindEvents(obj, [{
					"name": "WidgetAfterMoveRow",
					"code": " args.owner.testCodeCalled = true;"
				}],
				function() {}
			);
			obj.afterMoveRow();
			expect(obj.testCodeCalled).toBe(true);

		});

	it("bindEvents should use on when event is FieldOnTap or FieldOnLongTap", function() {
		var obj = {
			on: function(methodName, fn) {
				obj[methodName] = fn;
			}
		};
		var obj2 = {
			on: function(methodName, fn) {
				obj2[methodName] = fn;
			}
		};
		eventEngine.bindEvents(obj, [{
			"name": "FieldOnTap",
			"code": ";"
		}], function() {}, function() {});
		eventEngine.bindEvents(obj2, [{
			"name": "FieldOnLongTap",
			"code": ";"
		}], function() {}, function() {});

		expect(obj.tap).not.toEqual(undefined);
		expect(obj2.longtap).not.toEqual(undefined);
	});

	/*it("bindEvents should use touchstart instead of click when in mobile eviroment",function(){
		console.log(Zepto);
		Zepto = {};
		Zepto.os = {};

		Zepto.os.phone = true;//mock
		Zepto.os.tablet = true;//mock

		var obj = {};
		eventEngine.bindEvents(obj, [{ "name" : "FieldOnClick", "code" : ";" }], function(){}, function(){});
		expect(Zepto.os.phone).toEqual(true);
		expect(Zepto.os.tablet).toEqual(true);
		expect(obj.click).toEqual(undefined);
		expect(obj.touchstart).not.toEqual(undefined);
	});*/

	it("bindEvents should use click  when not in mobile eviroment", function() {
		Zepto.os.phone = false; //mock
		Zepto.os.tablet = false; //mock

		var obj = {};
		eventEngine.bindEvents(obj, [{
			"name": "FieldOnClick",
			"code": ";"
		}], function() {}, function() {});
		expect(Zepto.os.phone).toEqual(false);
		expect(Zepto.os.tablet).toEqual(false);
		expect(obj.click).not.toEqual(undefined);
	});

	it(
		"bindEvents should not call updateTemplate when method name in eventsWhenNotUpdateTemplate array ",
		function() {
			var obj = {};
			eventEngine.bindEvents(obj, [{
				"name": "onUpdate",
				"code": ";"
			}], function() {}, function() {});
			expect(updateTemplate).toEqual(false);
		});

	it(
		"bindEvents should call updateTemplate when method name not found in eventsWhenNotUpdateTemplate array ",
		function() {
			var obj = {};
			eventEngine.bindEvents(obj, [{
				"name": "WidgetAfterMoveRow",
				"code": ";"
			}], function() {}, function() {});
			obj.afterMoveRow();
			expect(updateTemplate).toEqual(true);
		});

	it("bindEvents should not bind event in object already binded", function() {
		var obj = {};
		eventEngine.bindEvents(obj, [{
			"name": "WidgetAfterMoveRow",
			"code": ";"
		}], function() {}, function() {});
		eventEngine.bindEvents(obj, [{
			"name": "FieldOnCancel",
			"code": ";"
		}], function() {}, function() {});

		expect(obj.afterMoveRow).not.toEqual(undefined);
		expect(obj.onCancel).toEqual(undefined);
	});

	it("bindEvents should bind business Event in obj.field when obj has property called field",
		function() {
			var obj = {
				field: {},
				off: function(methodName, fn) {
					if (fn) {
						fn();
					}
				},
				on: function(methodName, fn) {
					if (fn) {
						fn({
							isPropagationStopped: function() {
								return false;
							}
						});
					}
				}
			};
			eventEngine.bindEvents(obj, [{
				"name": "WidgetAfterMoveRow",
				"code": ";"
			}], function() {}, function() {});
			expect(obj.field).not.toEqual(undefined);
			expect(obj.field.afterMoveRow).not.toEqual(undefined);
		});

	// da um jeito de testar o __ignore__ e preventDefault
	it("bindEvents should when the name event is FieldOnClick dispare the event preventDefault",
		function() {
			var test = false;
			var event = {
				preventDefault: function() {
					test = true;
				},
				isPropagationStopped: function() {
					return false;
				},
				__ignore__: undefined
			};

			var obj = {
				off: function(methodName, fn) {
					delete obj[methodName];
				},
				on: function(methodName, fn) {
					obj[methodName] = fn;
				},
				field: {}
			};
			eventEngine.bindEvents(obj, [{
				"name": "FieldOnClick",
				"code": ";"
			}], function() {}, function() {});

			expect(test).toEqual(false);
			expect(obj.click).not.toEqual(undefined);

			obj.click(event);
			expect(test).toEqual(true);
		});

	// da um jeito de testar o __ignore__ e preventDefault
	it("bindEvents: When the name event is FieldOnClick and pass the key __ignore__ to event, preventDefault event should not be fired",
		function() {
			var test = false;
			var event = {
				preventDefault: function() {
					test = true;
				},
				isPropagationStopped: function() {
					return false;
				},
				__ignore__: true
			};

			var obj = {
				off: function(methodName, fn) {
					delete obj[methodName];
				},
				on: function(methodName, fn) {
					obj[methodName] = fn;
				},
				field: {}
			};
			eventEngine.bindEvents(obj, [{
				"name": "FieldOnClick",
				"code": ";"
			}], function() {}, function() {});

			expect(test).toEqual(false);
			expect(obj.click).not.toEqual(undefined);

			obj.click(event);
			expect(test).toEqual(false);
		});
	it("Event should not call the event with dispatch stoped!", function() {
		var obj = {
			i: 0
		};
		eventEngine.bindEvents(obj, [{
			"name": "WidgetAfterMoveRow",
			"code": "args.owner.i++;"
		}], function() {}, function() {});
		obj.afterMoveRow();
		expect(obj.i).toEqual(1);
		eventEngine.stopDispatchEvent("Tests!!");
		expect(eventEngine.isDispatchEventStopped()).toEqual(true);
		obj.afterMoveRow();
		expect(obj.i).toEqual(1);
		eventEngine.restoreDispatchEvent();
		obj.afterMoveRow();
		expect(obj.i).toEqual(2);
	});

	it("StopDispatchEvent should throw and error when reason is undefined!", function() {
		expect(function() {
			eventEngine.stopDispatchEvent();
		}).toThrow();
	});

	it("registerEvent  should throw and when event is already mapped!!", function() {
		expect(function() {
			eventEngine.registerEvent("WidgetOnEnter", "onEnter");
		}).toThrow();
	});

	it("registerEvent  should throw and when parameter are invalids!!", function() {
		expect(function() {
			eventEngine.registerEvent("WidgetOnEnter");
		}).toThrow();
		expect(function() {
			eventEngine.registerEvent(null, "onEnter");
		}).toThrow();
	});

	it("unregisterEvent unregister evens from eventEngine!!", function() {
		expect(function() {
			eventEngine.unregisterEvent("WidgetOnEnter");
			eventEngine.registerEvent("WidgetOnEnter", "onEnter");
		}).not.toThrow();
	});
	
	it("unbindEvent should remove the event from the listeners",
		function() {
			var obj = {};
			eventEngine.bindEvent(obj, "WidgetAfterMoveRow", ";");
			expect(obj.afterMoveRow).not.toEqual(undefined);
			expect(obj.afterMoveRowlistener.length).toBe(1);
			
			var unbindResult = eventEngine.unbindEvent(obj, "WidgetAfterMoveRow", 0);
			expect(unbindResult).toBeTruthy();
			expect(obj.afterMoveRowlistener.length).toBe(0);
	});
});